/*
 *   Lessfs: A data deduplicating filesystem.
 *   Copyright (C) 2008 Mark Ruijter <mruijter@lessfs.com>
 *
 *   This program is free software;  you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program;  if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <unistd.h>
#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <semaphore.h>
#include <pthread.h>
#include <fuse.h>

#include <tcutil.h>
#include <tchdb.h>
#include <tcbdb.h>
#include <stdlib.h>
#include <stdbool.h>

#include "lib_log.h"
#include "lib_safe.h"
#include "lib_cfg.h"
#include "lib_str.h"
#include "retcodes.h"
#ifdef LZO
#include "lib_lzo.h"
#else
#include "lib_qlz.h"
#endif
#include "lib_tc.h"

#ifdef ENABLE_CRYPTO
unsigned char *passwd=NULL;
#endif

#define die_dataerr(f...) { LFATAL(f); exit(EXIT_DATAERR); }

char *logname;
char *function;
int debug = 0;
struct configdata *config;

int BLKSIZE = 0;
int max_threads;

extern TCHDB *dbb;
extern TCHDB *dbu;
extern TCHDB *dbp;
extern TCBDB *dbl;
extern TCHDB *dbs;
extern TCHDB *dbdta;
extern TCBDB *dbdirent;

BLKDTA **tdta = NULL;

void usage(char *name)
{
    printf("Usage: %s /path_to_config.cfg\n", name);
    exit(-1);
}

int main(int argc, char *argv[])
{
    char *dbpath;
    char *dbg = NULL;
#ifdef ENABLE_CRYPTO
    int rnd;
    char *ckpasswd;
#endif
    struct stat stbuf;
    char *p;

    if (argc < 2)
        usage(argv[0]);

    dbg = getenv("DEBUG");
    if (NULL != dbg)
        debug = atoi(dbg);

    FUNC;
    if (-1 == r_env_cfg(argv[1]))
        usage(argv[0]);
    config = s_malloc(sizeof(struct configdata));
    config->blockdata = read_val("BLOCKDATA_PATH");
    config->blockdatabs = read_val("BLOCKDATA_BS");
    config->blockusage = read_val("BLOCKUSAGE_PATH");
    config->blockusage = read_val("BLOCKUSAGE_PATH");
    config->blockusagebs = read_val("BLOCKUSAGE_BS");
    config->dirent = read_val("DIRENT_PATH");
    config->direntbs = read_val("DIRENT_BS");
    config->fileblock = read_val("FILEBLOCK_PATH");
    config->fileblockbs = read_val("FILEBLOCK_BS");
    config->meta = read_val("META_PATH");
    config->metabs = read_val("META_BS");
    config->hardlink = read_val("HARDLINK_PATH");
    config->hardlinkbs = read_val("HARDLINK_BS");
    config->symlink = read_val("SYMLINK_PATH");
    config->symlinkbs = read_val("SYMLINK_BS");
    config->encryptdata=0;
    config->passwd=NULL;
    config->iv=NULL;
#ifdef ENABLE_CRYPTO
    config->iv=s_malloc(8);
    memset(config->iv,0,8);
    config->encryptdata=0;
    p = read_val("ENCRYPT_DATA");
    if ( NULL != p ) {
        if ( 0 == strcasecmp(p,"ON")) config->encryptdata=1;
    }
#endif
    dbpath = as_sprintf("%s/blockdata.tch", config->blockdata);
    if (-1 != stat(dbpath, &stbuf)) {
        fprintf(stderr,
                "Data %s already exists, please remove it and try again\n",
                dbpath);
        exit(EXIT_DATAERR);
    }
    config->encryptmeta=1;
#ifdef ENABLE_CRYPTO
    if ( config->encryptdata ) { 
       rnd=s_open("/dev/random", O_RDONLY);
       if ( 8 != fullRead(rnd,config->iv,8 )) die_dataerr("Could not read 8 bytes from /dev/random"); 
       config->passwd=(unsigned char *)s_strdup(getpass("Password: "));
       ckpasswd=s_strdup(getpass("Re-Enter Password: "));
       if ( 0 != strcmp((char *)config->passwd,ckpasswd)){
          fprintf(stderr,"Password values do not match.\n");
          exit(EXIT_SYSTEM);
       }
       free(ckpasswd);
       p=getenv("ENCRYPT_META");
       if ( NULL != p ) {
          if ( 0 != strcasecmp(p,"ON")){
             LINFO("Metadata encryption is off");
             config->encryptmeta=0;
          }
       }
    }
#endif
    formatfs();
    sync();
#ifdef ENABLE_CRYPTO   
    if ( config->encryptdata ) free(config->passwd);
    free(config->iv);
#endif
    free(config);
    exit(0);
}
